package com.iot.smart.water.meter.service.Impl;

import com.iot.smart.water.meter.dao.DataMapper;
import com.iot.smart.water.meter.dao.MeterMapper;
import com.iot.smart.water.meter.dao.VolumeMapper;
import com.iot.smart.water.meter.model.*;
import com.iot.smart.water.meter.service.DataService;
import com.iot.smart.water.meter.util.DateUtil;
import com.iot.smart.water.meter.util.line.LineNotify;
import com.iot.smart.water.meter.util.WeekUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.scheduling.annotation.Scheduled;

@Service
@EnableScheduling
public class DataServiceImpl implements DataService {

    private static final Logger logger = LoggerFactory.getLogger(DataServiceImpl.class);

    @Autowired
    private DataMapper dataMapper;

    @Autowired
    private MeterMapper meterMapper;

    @Autowired
    private VolumeMapper volumeMapper;

    @Autowired
    private LineNotify lineNotify;

    @Override
    public Data getLatestData(String meterName, String start, String end) {
        return dataMapper.selectLatestDataInMonthByName(meterName, start, end);
    }

    @Override
    public List<DailyData> getDailyData(String meterName, String date) {
        Date parseDate = DateUtil.parseDate(date, "yyyy-MM-dd");
        if (parseDate == null) {
            return null;
        }
        List<DailyData> list = new ArrayList<>();
        for (int i = 0; i < 24; i++) {

            DailyData dailyData = new DailyData();
            String start;
            String end;
            if (i < 10) {
                dailyData.setHour(" 0" + i + ":00");
                start = date + " 0" + i + ":00:00";
                end = date + " 0" + i + ":59:59";
            } else {
                dailyData.setHour(" " + i + ":00");
                start = date + " " + i + ":00:00";
                end = date + " " + i + ":59:59";
            }
            Data data = getLatestData(meterName, start, end);
            dailyData.setTotalMilliters(data == null ? 0 : data.getTotalMilliters());
            list.add(dailyData);
        }
        return list;
    }

    @Override
    public List<MonthlyData> getMonthlyData(String meterName, String date) {
        Date parseDate = DateUtil.parseDate(date, "yyyy-MM-dd");
        if (parseDate == null) {
            return null;
        }
        List<MonthlyData> list = new ArrayList<>();
        int day = DateUtil.getDay(parseDate);
        int maxDay = DateUtil.getDaysOfMonth(parseDate);
        String start;
        String end;
        for (int i = 1; i <= maxDay; i++) {
            MonthlyData monthlyData = new MonthlyData();
            if (i < 10) {
                monthlyData.setDay("0" + i);
            } else {
                monthlyData.setDay(String.valueOf(i));
            }
            String dayDate = DateUtil.formatDiffDate(parseDate, i - day, "yyyy-MM-dd");
            start = dayDate + " 00:00:00";
            end = dayDate + " 23:59:59";
            Data data = getLatestData(meterName, start, end);
            monthlyData.setTotalMilliters(data == null ? 0 : data.getTotalMilliters());
            list.add(monthlyData);
        }
        return list;
    }

    @Override
    public List<WeeklyData> getWeeklyData(String meterName, String date) {
        Date parseDate = DateUtil.parseDate(date, "yyyy-MM-dd");
        if (parseDate == null) {
            return null;
        }
        List<WeeklyData> list = new ArrayList<>();
        int week = DateUtil.getWeekDay(parseDate);
        String start;
        String end;
        for (int i = 1; i <= 7; i++) {
            WeeklyData weeklyData = new WeeklyData();
            weeklyData.setWeek(WeekUtil.WEEK_CONTENT[i]);
            String weekDate = DateUtil.formatDiffDate(parseDate, i - week, "yyyy-MM-dd");
            start = weekDate + " 00:00:00";
            end = weekDate + " 23:59:59";
            Data data = getLatestData(meterName, start, end);
            weeklyData.setTotalMilliters(data == null ? 0 : data.getTotalMilliters());
            list.add(weeklyData);
        }
        return list;
    }

    @Scheduled(cron = "0 * * * * ?")
    public void scheduleTask() {
        logger.info("DataServiceImpl schedule task");
        List<Volume> volumes = volumeMapper.selectAllVolume();
        if (volumes != null) {
            for (Volume volume : volumes) {
                if (volume.getNotifyLimit() == 0) {
                    Meter meter = meterMapper.selectMeterById(volume.getMeter_id());
                    Data data = dataMapper.selectLatestDataByName(meter.getMeterName());
                    if (data !=null) {
                        if (volume.getVolume() > 0 && data.getTotalMilliters() >= volume.getVolume()) {
                            volume.setNotifyLimit(1);
                            volumeMapper.updateVolume(volume);
                            lineNotify.notifyMe("This Week water exceeds the limit volume is " + volume.getVolume() + " milliliter", 16, 1);
                        }
                    }
                }
            }
        }
    }

}


